# coding: utf-8

import datetime
import unittest

import fastobo

# --- TermFrame --------------------------------------------------------------

class TestTermFrame(unittest.TestCase):

    type = fastobo.term.TermFrame

    def setUp(self):
        self.id = fastobo.id.PrefixedIdent("MS", "1000031")

    def test_init(self):
        try:
            frame = self.type(self.id)
        except Exception:
            self.fail("could not create `TermFrame` instances")

    def test_init_iterable(self):
        try:
            frame = self.type(self.id, [])
        except Exception:
            self.fail("could not create `TermFrame` instances")
        try:
            frame = self.type(self.id, [
                fastobo.term.NameClause("thing"),
                fastobo.term.CreatedByClause("Martin Larralde")
            ])
        except Exception:
            self.fail("could not create `TermFrame` from iterable")

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type, 1)
        self.assertRaises(TypeError, self.type, [1])
        self.assertRaises(TypeError, self.type, ["abc"])
        self.assertRaises(TypeError, self.type, "abc")
        self.assertRaises(TypeError, self.type, self.id, 1)
        self.assertRaises(TypeError, self.type, self.id, [1])
        self.assertRaises(TypeError, self.type, self.id, ["abc"])
        self.assertRaises(TypeError, self.type, self.id, "abc")


# --- ConsiderClause ---------------------------------------------------------

class TestConsiderClause(unittest.TestCase):

    type = fastobo.term.ConsiderClause

    def setUp(self):
        self.id = fastobo.id.PrefixedIdent("MS", "1000031")
        self.id2 = fastobo.id.PrefixedIdent("MS", "1000032")

    def test_init(self):
        try:
            frame = self.type(self.id)
        except Exception:
            self.fail("could not create `ConsiderClause` instances")

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type)
        self.assertRaises(TypeError, self.type, 1)

    def test_eq(self):
        self.assertEqual(self.type(self.id), self.type(self.id))
        self.assertNotEqual(self.type(self.id), self.type(self.id2))


# --- IsObsoleteClause -------------------------------------------------------


class TestIsObsoleteClause(unittest.TestCase):

    type = fastobo.term.IsObsoleteClause

    def test_init(self):
        try:
            frame = self.type(True)
        except Exception:
            self.fail("could not create `IsObsoleteClause` instances")

    def test_property_obsolete(self):
        c = self.type(False)
        self.assertEqual(c.obsolete, False)
        c.obsolete = True
        self.assertEqual(c.obsolete, True)

    def test_repr(self):
        self.assertEqual(repr(self.type(False)), "IsObsoleteClause(False)")
        self.assertEqual(repr(self.type(True)), "IsObsoleteClause(True)")

    def test_str(self):
        self.assertEqual(str(self.type(False)), "is_obsolete: false")
        self.assertEqual(str(self.type(True)), "is_obsolete: true")

    def test_eq(self):
        self.assertEqual(self.type(True), self.type(True))
        self.assertEqual(self.type(False), self.type(False))
        self.assertNotEqual(self.type(False), self.type(True))



# coding: utf-8

import os
import unittest

import fastobo

MS = os.path.realpath(os.path.join(__file__, "..", "data", "ms.obo"))


class TestLoad(unittest.TestCase):

    def test_file_not_found(self):
        self.assertRaises(FileNotFoundError, fastobo.load, "abcdef")

    def test_type_error(self):
        self.assertRaises(TypeError, fastobo.load, 1)
        self.assertRaises(TypeError, fastobo.load, [])

        with open(MS) as f:
            self.assertRaises(TypeError, fastobo.load, f)

    def test_error_propagation(self):

        def read(x):
            if x == 0:
                return b''
            raise RuntimeError(x)

        with open(MS, 'rb') as f:
            f.read = read
            self.assertRaises(RuntimeError, fastobo.load, f)

    def test_syntax_error(self):
        self.assertRaises(SyntaxError, fastobo.loads, "hello there")


class TestIter(unittest.TestCase):

    def test_file_not_found(self):
        self.assertRaises(FileNotFoundError, fastobo.iter, "abcdef")

    def test_type_error(self):
        self.assertRaises(TypeError, fastobo.iter, 1)
        self.assertRaises(TypeError, fastobo.iter, [])

        with open(MS) as f:
            self.assertRaises(TypeError, fastobo.iter, f)



# coding: utf-8

import datetime
import unittest

import fastobo


# -- OboDoc ------------------------------------------------------------------

class TestOboDoc(unittest.TestCase):

    type = fastobo.doc.OboDoc

    def setUp(self):
        self.header = fastobo.header.HeaderFrame([
            fastobo.header.FormatVersionClause("1.4"),
            fastobo.header.SavedByClause("Martin Larralde"),
        ])
        self.entities = [
            fastobo.term.TermFrame(fastobo.id.PrefixedIdent("MS", "1000031")),
            fastobo.typedef.TypedefFrame(fastobo.id.UnprefixedIdent("part_of"))
        ]

    def test_init(self):
        try:
            doc = self.type()
        except Exception:
            self.fail("could not create `OboDoc` instances")
        self.assertEqual(len(doc.header), 0)
        self.assertEqual(len(doc), 0)

    def test_init_header(self):
        try:
            doc = self.type(self.header)
        except Exception:
            self.fail("could not create `OboDoc` instances with a header")
        self.assertEqual(len(doc.header), 2)
        self.assertEqual(doc.header[0], self.header[0])
        self.assertEqual(doc.header[1], self.header[1])
        self.assertEqual(len(doc), 0)

    def test_init_entities(self):
        try:
            doc = self.type(entities=self.entities)
        except Exception:
            self.fail("could not create `OboDoc` instances with a header")
        self.assertEqual(len(doc.header), 0)
        self.assertEqual(len(doc), 2)
        self.assertEqual(doc[0], self.entities[0])
        self.assertEqual(doc[1], self.entities[1])

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type, 1)
        self.assertRaises(TypeError, self.type, [1])
        self.assertRaises(TypeError, self.type, ["abc"])
        self.assertRaises(TypeError, self.type, "abc")
        self.assertRaises(TypeError, self.type, self.header, 1)
        self.assertRaises(TypeError, self.type, self.header, [1])
        self.assertRaises(TypeError, self.type, self.header, ["abc"])
        self.assertRaises(TypeError, self.type, self.header, "abc")
        self.assertRaises(TypeError, self.type, 1, self.entities)
        self.assertRaises(TypeError, self.type, [1], self.entities)
        self.assertRaises(TypeError, self.type, ["abc"], self.entities)
        self.assertRaises(TypeError, self.type, "abc", self.entities)



#!/usr/bin/env python3

import configparser
import os

import setuptools
import setuptools_rust as rust
from setuptools.command.sdist import sdist as _sdist


class sdist(_sdist):

    def run(self):
        # build `pyproject.toml` from `setup.cfg`
        c = configparser.ConfigParser()
        c.add_section("build-system")
        c.set("build-system", "requires", str(self.distribution.setup_requires))
        c.set("build-system", 'build-backend', '"setuptools.build_meta"')
        with open("pyproject.toml", "w") as pyproject:
            c.write(pyproject)

        # run the rest of the packaging
        _sdist.run(self)


setuptools.setup(
    setup_requires=["setuptools", "setuptools_rust"],
    cmdclass=dict(sdist=sdist),
    rust_extensions=[rust.RustExtension(
        "fastobo",
        path="Cargo.toml",
        binding=rust.Binding.PyO3,
        strip=rust.Strip.Debug,
        features=["extension-module"],
    )],
)
